var BrowserApi = {
    list: {},
    methods: {},
    init: function(config) {
        //load interface for browser
        var iApi = new iBrowserApi('browserapi', [
            'getStorage', // chrome.storage.local.get(key, callback);
            'setStorage', // chrome.storage.local.set(obj, callback);
            'clearStorage', // chrome.storage.local.clear();
            'removeStorage', // chrome.storage.local.remove(key, callback);
            'onStorageChange', // chrome.storage.onChanged
            'getUrl', // chrome.runtime.getURL(path);
            'getBackgroundPage', // chrome.runtime.getBackgroundPage(callback);
            'getAuthToken', // chrome.identity.getAuthToken(obj, callback);
            'onMessage', // chrome.runtime.onMessage;
            'onInstalled', // chrome.runtime.onInstalled;
            'onCookieChange', // chrome.cookies.onChanged;
            'onBrowserActionClick', // chrome.browserAction.onClicked;
            'setUninstallUrl', // chrome.runtime.setUninstallUrl(link);
            'getAllCookies', // chrome.cookies.getAll(obj, callback);
            'createTab', // chrome.tabs.create(obj, callback);
            'getTab', // chrome.tabs.get(obj, callback);
            'insertTabCss', // chrome.tabs.insertCSS(id, obj);
            'executeTabScript', // chrome.tabs.executeScript(id, obj);
            'onTabCreated', // chrome.tabs.onCreated;
            'onTabUpdated', // chrome.tabs.onUpdated;
            'sendMessage', // chrome.runtime.sendMessage(message, callback);
            'onNotificationClose', // chrome.notifications.onClosed;
            'onNotificationButtonClicked', // chrome.notifications.onButtonClicked;
            'createNotification', // chrome.notifications.create(opt, callback);
            'clearNotifications', // chrome.notifications.clear(id);
            'getManifest', // chrome.app.getDetails();
            'getLastError' // chrome.runtime.lastError;
            
        ]);

        iBrowserApi.ensureImplements(config.apiMethods, iApi);

        BrowserApi.methods = config.apiMethods;
    },
    getConfig: function(browserName){
        switch(browserName){
            case 'Chrome':
            case 'Opera':
            case 'Yandex':
            case 'Chromium':
                var config = {
                    name: browserName,
                    apiMethods: {
                        getStorage: function(key, callback) {
                            return chrome.storage.local.get(key, callback);
                        },
                        setStorage: function(obj, callback) {
                            return chrome.storage.local.set(obj, callback);
                        },
                        removeStorage: function(key, callback) {
                            return chrome.storage.local.remove(key, callback);
                        },
                        clearStorage: function() {
                            return chrome.storage.local.clear();
                        },
                        onStorageChange: function() {
                            return chrome.storage.onChanged;
                        }, 
                        getUrl: function(path){
                            return chrome.runtime.getURL(path);
                        },
                        getBackgroundPage: function(callback) {
                            return chrome.runtime.getBackgroundPage(callback);
                        },
                        getAuthToken: function(obj, callback) {
                            return chrome.identity.getAuthToken(obj, callback);
                        },
                        onMessage: function() {
                            return chrome.runtime.onMessage;
                        },
                        onInstalled: function() {
                            return chrome.runtime.onInstalled;
                        },
                        onCookieChange: function() {
                            return chrome.cookies.onChanged;
                        },
                        onBrowserActionClick: function() {
                            return chrome.browserAction.onClicked;
                        },
                        setUninstallUrl: function(link) {
                            return chrome.runtime.setUninstallURL(link);
                        },
                        getAllCookies: function(obj, callback) {
                            return chrome.cookies.getAll(obj, callback);
                        },
                        createTab: function(obj, callback) {
                            return chrome.tabs.create(obj, callback);
                        },
                        getTab: function(obj, callback) {
                            return chrome.tabs.get(obj, callback);
                        },
                        insertTabCss: function(id, obj) {
                            return chrome.tabs.insertCSS(id, obj);
                        },
                        executeTabScript: function(id, obj) {
                            return chrome.tabs.executeScript(id, obj);
                        },
                        onTabCreated: function() {
                            return chrome.tabs.onCreated;
                        },
                        onTabUpdated: function() {
                            return chrome.tabs.onUpdated;
                        },
                        sendMessage: function(message, callback) {
                            return chrome.runtime.sendMessage(message, callback);
                        },
                        onNotificationClose: function() {
                            return chrome.notifications.onClosed;
                        },
                        onNotificationButtonClicked: function() {
                            return chrome.notifications.onButtonClicked;
                        },
                        createNotification: function(opt, callback){
                            return chrome.notifications.create(opt, callback);
                        },
                        clearNotifications: function(id) {
                            return chrome.notifications.clear(id);
                        },
                        getManifest: function() {
                            return chrome.app.getDetails();
                        },
                        getLastError: function() {
                            return chrome.runtime.lastError;
                        }
                    }
                };
                return config;
            case "Firefox":
                var config = {
                    name: browserName,
                    apiMethods: {
                        getStorage: function(key, callback) {
                            return browser.storage.local.get(key, callback);
                        },
                        setStorage: function(obj, callback) {
                            return browser.storage.local.set(obj, callback);
                        },
                        removeStorage: function(key, callback) {
                            return browser.storage.local.remove(key, callback);
                        },
                        clearStorage: function() {
                            return browser.storage.local.clear();
                        },
                        onStorageChange: function() {
                            return browser.storage.onChanged;
                        }, 
                        getUrl: function(path){
                            return browser.runtime.getURL(path);
                        },
                        getBackgroundPage: function(callback) {
                            return browser.runtime.getBackgroundPage(callback);
                        },
                        getAuthToken: function(obj, callback) {
                            return browser.identity.getAuthToken(obj, callback);
                        },
                        onMessage: function() {
                            return browser.runtime.onMessage;
                        },
                        onInstalled: function() {
                            return browser.runtime.onInstalled;
                        },
                        onCookieChange: function() {
                            return browser.cookies.onChanged;
                        },
                        onBrowserActionClick: function() {
                            return browser.browserAction.onClicked;
                        },
                        setUninstallUrl: function(link) {
                            return browser.runtime.setUninstallURL(link);
                        },
                        getAllCookies: function(obj, callback) {
                            return browser.cookies.getAll(obj, callback);
                        },
                        createTab: function(obj, callback) {
                            return browser.tabs.create(obj, callback);
                        },
                        getTab: function(obj, callback) {
                            return browser.tabs.get(obj, callback);
                        },
                        insertTabCss: function(id, obj) {
                            return browser.tabs.insertCSS(id, obj);
                        },
                        executeTabScript: function(id, obj) {
                            return browser.tabs.executeScript(id, obj);
                        },
                        onTabCreated: function() {
                            return browser.tabs.onCreated;
                        },
                        onTabUpdated: function() {
                            return browser.tabs.onUpdated;
                        },
                        sendMessage: function(message, callback) {
                            return browser.runtime.sendMessage(message, callback);
                        },
                        onNotificationClose: function() {
                            return browser.notifications.onClosed;
                        },
                        onNotificationButtonClicked: function() {
                            return browser.notifications.onButtonClicked;
                        },
                        createNotification: function(opt, callback){
                            return browser.notifications.create(opt, callback);
                        },
                        clearNotifications: function(id) {
                            return browser.notifications.clear(id);
                        },
                        getManifest: function() {
                            return browser.runtime.getManifest();
                        },
                        getLastError: function() {
                            return browser.runtime.lastError;
                        }
                    }
                };
                return config;
        }
    } 
};
